==> migrate-1.0.1.src.tar.gz: migrate-1.0.1/src/gammalike.c <==
// gamma deviated mutation rate among loci
//
//
// code for the calculation of the gamma likelihood
// and for its derivatives
#include "migration.h"
#include "broyden.h"
#include "combroyden.h"


double gamma_loci_like(nr_fmt *nr, double denom, double *oparam, double *olparam);
double gamma_locus_like(nr_fmt *nr, double *oparam, double *olparam,
			long locus, double denom);
double loggf(double r, double alpha, double theta1);
void gamma_loci_derivative(helper_fmt *helper);
void gamma_locus_derivative(helper_fmt *helper, long locus);
double gamma_theta1_derivative(double theta, double *param, nr_fmt *nr,
			       long locus, int *sign);
double gamma_other_derivative(long which, double theta, double *param, 
			      nr_fmt *nr,long locus, int *sign);
double gamma_alpha_derivative(double theta, double *param, nr_fmt *nr, 
			      long locus, int *sign);
void gamma_parts_gradient(double x, double *param, nr_fmt *nr, long locus);
double dt_locus_gsum (long which, double x, double *param, nr_fmt * nr,
		      timearchive_fmt ** atl, long locus);


double gamma_loci_like(nr_fmt *nr, double denom, double *oparam, double *olparam)
{
  long locus;
  double summ=0.0;
  for(locus=0;locus<nr->world->loci;locus++)
    {
      if (nr->skiploci[locus])
	continue;
      nr->locilikes[locus] = gamma_locus_like(nr,oparam, olparam, locus, denom);      summ += nr->locilikes[locus];
    }
  return summ;
}

double gamma_locus_like(nr_fmt *nr, double *oparam, double *olparam, 
			long locus, double denom)
{
  long r;
  double tempmax = -DBL_MAX;
  double *temp;
  double summ=0.;
  double alpha = oparam[nr->numpop2];
  double theta1 = oparam[0];
  temp = (double *) calloc(GAMMA_INTERVALS, sizeof(double));
  for (r=0;r<GAMMA_INTERVALS;r++)
    {
      set_gamma_param (nr->param, oparam, 
		       nr->lparam, olparam, nr->rate[r], nr);
      temp[r] = nr->probcat[r] + loggf(nr->rate[r], alpha, theta1) - denom +
	(*calc_like)(nr, nr->param, nr->lparam, locus);
      //      printf("%f ",temp[r]);
      if(temp[r]>tempmax)
	tempmax = temp[r];
    }
  //  printf("\n");
  for (r=0; r<GAMMA_INTERVALS; r++)
    summ += exp(temp[r] - tempmax);
  free(temp);
  return log(summ) + tempmax;
}

double loggf(double r, double alpha, double theta1)
{
  return -r * alpha / theta1 + log(r) * (alpha-1.);
}

void gamma_loci_difference(helper_fmt *helper)
{
  long i;
  nr_fmt *nr = helper->nr;
  long size = nr->partsize;
  double mle, high, low, param1;
  double *lparam;
  double *param;
  double *xv = helper->xv;
  double *expxv = helper->expxv;
  double epsilon=EPSILON;
  //  double logepsilon = log(epsilon);
  memset(nr->d,0, size*sizeof(double));
  setdoublevec1d(&lparam, helper->xv, size);
  setdoublevec1d(&param, helper->expxv, size);
  mle = gamma_loci_like(nr, helper->weight, param, lparam);
  for(i=0;i<size;i++)
    {
      memcpy (param, expxv, sizeof (double) * size);
      memcpy (lparam, xv, sizeof (double) * size);
      param1 = (param[i] += epsilon);
      lparam[i] = log(param[i]);
      high = gamma_loci_like(nr, helper->weight, param, lparam);
      param[i] -= 2. * epsilon;
      if(param[i]<0.0)
	{
	  low = mle;
	  param[i]=expxv[i];
	}
      else
	{
	  lparam[i] = log(param[i]);
	  low = gamma_loci_like(nr, helper->weight, param, lparam);
	}
      nr->d[i] = (high - low)/(param1 - param[i]);
    }
  free(lparam);
  free(param);
}

void gamma_loci_derivative(helper_fmt *helper)
{
  long locus;
  memset(helper->nr->d,0,helper->nr->partsize*sizeof(double));
  for(locus=0; locus<helper->nr->world->loci; locus++)
    {
      if (helper->nr->skiploci[locus])
	continue;
      //adds up in nr->d
      gamma_locus_derivative(helper,locus);
    }
}

void  gamma_locus_derivative(helper_fmt *helper, long locus)
{
  long r, i, ii,z;
  double **temp, *tempmax;
  double ll;
  double sumi;
  int **stemp;
  //double *summll;
  //double summllsum;
  //double summllmax;
  double rate;
  double lograte;
  double tmp;  
  double denom = helper->weight;
  nr_fmt *nr = helper->nr;
  double alpha = helper->expxv[nr->numpop2];
  double theta1 = helper->expxv[0];
  long partsize = nr->partsize;
  //summll = (double *) calloc(GAMMA_INTERVALS, sizeof(double));
  tempmax = (double *) calloc(partsize, sizeof(double));
  temp = (double **) calloc(partsize, sizeof(double*));
  temp[0] = (double *) calloc(partsize * GAMMA_INTERVALS, sizeof(double));
  stemp = (int **) calloc(partsize, sizeof(int*));
  stemp[0] = (int *) calloc(partsize * GAMMA_INTERVALS, sizeof(int));
  tempmax[0]= -DBL_MAX;
  for (r=1;r<partsize;r++)
    {
      temp[r] = temp[0] + r * GAMMA_INTERVALS;  
      stemp[r] = stemp[0] + r * GAMMA_INTERVALS;
      tempmax[r] = -DBL_MAX;
    }
  //summllmax = -DBL_MAX;
  //summllsum=0.0;
  for (r=0;r<GAMMA_INTERVALS;r++)
    {
      rate = nr->rate[r];
      lograte = log(rate);
      set_gamma_param (nr->param, helper->expxv, 
		       nr->lparam, helper->xv, rate, nr);
      ll = nr->probcat[r] + (*calc_like) (nr, nr->param, nr->lparam, locus);
      ll += -rate * alpha / theta1  + lograte * (alpha - 1.) - denom;
      //summll[r] = ll;
      //if(ll>summllmax)
      //summllmax = ll;
      gamma_parts_gradient(rate, helper->expxv, nr, locus);//changes nr->parts
      //first of the derivatives: theta1
      tmp = rate * alpha / (theta1 * theta1) + nr->parts[0];
      stemp[0][r] =  tmp < 0. ? -1 : 1;
      temp[0][r] = ll + log(fabs(tmp));
      if(tempmax[0] < temp[0][r])
      	tempmax[0] = temp[0][r];
      //all others except alpha
      for(i=1;i<nr->numpop2;i++)
      {
	stemp[i][r] = (nr->parts[i] < 0.) ? -1 : 1; 
	temp[i][r] =  ll + log(fabs(nr->parts[i]));
	if(tempmax[i] < temp[i][r])
	  tempmax[i] = temp[i][r];
      }
      // last derivative: alpha
      tmp =  - rate / theta1 + lograte;
      stemp[i][r] =  tmp < 0. ? -1 : 1;
      temp[i][r] = ll + log(fabs(tmp));
      if(tempmax[i] < temp[i][r])
	tempmax[i] = temp[i][r];
    }
  //for (r=0;r<GAMMA_INTERVALS;r++)
  //  {
  //    summllsum += exp(summll[r] - summllmax);
  //  }
  //summllsum = log(summllsum) + summllmax; 
  z=0;
  for(ii=0; ii < nr->partsize - nr->profilenum; ii++)
    {
      i = (nr->profilenum > 0) ? nr->indeks[z++] : ii; 
      sumi = 0;
      for (r=0;r<GAMMA_INTERVALS;r++)
	sumi += ((double) stemp[i][r]) *  exp(temp[i][r] - tempmax[i] );
      if(i==0)
	{
	  //  printf("summll=%f locilikes[%li]=%f\n",summllsum,locus,nr->locilikes[locus]);
	  sumi = sumi * exp(tempmax[i] - nr->locilikes[locus]) - alpha/theta1;
	}
      else 
	{
	  if(i==nr->numpop2)
	    {
	      sumi = sumi * exp(tempmax[i] - nr->locilikes[locus])
		- polygamma(0, alpha) 
		- log(theta1/alpha) + 1.;
	    }
	  else
	    sumi = sumi * exp(tempmax[i] - nr->locilikes[locus]);
	}
      nr->d[ii] += sumi;  
    }
  free(temp[0]);
  free(temp);
  free(stemp[0]);
  free(stemp);
  //free(summll);
}

void gamma_parts_gradient(double x, double *param, nr_fmt *nr, long locus)
{
  long g;

  //  double copysum=0;
  long pop, i;
  double expapg;//, summ;
  //double summm = 0.0;
  long msta, msto;
  double waitmig;
  double pointmig;
  tarchive_fmt *tl;
  double *part;//, df = 0.0;
  //  double value ;
  double theta1 = param[0];
  double *geo = nr->world->data->geo;
  long rep;
  double theta_ratio = theta1 / x;
  part = (double *) malloc(sizeof(double)*nr->partsize);
  memset(nr->parts, 0, sizeof(double)*nr->partsize);
  for (rep = nr->repstart; rep < nr->repstop; rep++)
    {
      for (g = 0; g < nr->atl[rep][locus].T; g++)
	{
	  if (nr->apg[rep][locus][g] > -40.)
	    {
	      tl = nr->atl[rep][locus].tl;
	      //copies = (g > 0) ? tl[g].copies : tl[g].copies - 1;
	      expapg = /*copies  */ exp (nr->apg[rep][locus][g]);
	      if(expapg==0.0)
		continue;
	      //      summm += expapg;
	      // population 1 is treated differently
	      part[0] = 0.0;
	      msta = nr->mstart[0];
	      msto = nr->mend[0];
	      for (i = msta; i < msto; i++)
		{
		  part[0] += -geo[i] * param[i] * tl[g].km[0] / x + 
		    tl[g].mindex[i] / theta1;
		  if (param[i] > 0.)
		    part[i] = ((tl[g].mindex[i] / param[i]) 
			       - geo[i] * tl[g].km[0] 
			       * theta_ratio);
		}
	      // all other populations
	      for (pop = 1; pop < nr->numpop; pop++)
		{
		  part[pop] = -tl[g].p[pop] / param[pop] + 
		    tl[g].kt[pop] * theta1 / (x * param[pop] * param[pop]);
		  
		  msta = nr->mstart[pop];
		  msto = nr->mend[pop];
		  //		  z = 0;
		  waitmig = 0;
		  pointmig = 0;
		  for (i = msta; i < msto; i++)
		    {
		      waitmig += geo[i] * param[i];
		      pointmig += tl[g].mindex[i];
		      if (param[i] > 0.)
			part[i] = ((tl[g].mindex[i] / param[i]) 
				   - geo[i] * tl[g].km[pop] 
				   * theta_ratio);
		    }
		  part[0] += pointmig/theta1 - waitmig * tl[g].km[pop] / x +
		    tl[g].p[pop] / theta1 - tl[g].kt[pop]/(param[pop]*x);
		}
	      for(pop=0;pop<nr->world->numpop2;pop++)
		nr->parts[pop] += expapg * part[pop]; 
	    }
	}
    }
  for(pop=0;pop<nr->world->numpop2;pop++)
    nr->parts[pop] /= nr->PGC[locus]; 
  // printf("nr->PGC[%li]=%f == %f\n",locus,nr->PGC[locus],summm);
  free(part);
}

==> migrate-1.1.src.tar.gz: migrate-1.1/src/gammalike.c <==
// gamma deviated mutation rate among loci
//
//
// code for the calculation of the gamma likelihood
// and for its derivatives
#include "migration.h"
#include "broyden.h"
#include "combroyden.h"


double gamma_loci_like(nr_fmt *nr, double denom, double *oparam, double *olparam);
double gamma_locus_like(nr_fmt *nr, double *oparam, double *olparam,
			long locus, double denom);
double loggf(double r, double alpha, double theta1);
void gamma_loci_derivative(helper_fmt *helper);
void gamma_locus_derivative(helper_fmt *helper, long locus);
double gamma_theta1_derivative(double theta, double *param, nr_fmt *nr,
			       long locus, int *sign);
double gamma_other_derivative(long which, double theta, double *param, 
			      nr_fmt *nr,long locus, int *sign);
double gamma_alpha_derivative(double theta, double *param, nr_fmt *nr, 
			      long locus, int *sign);
void gamma_parts_gradient(double x, double *param, nr_fmt *nr, long locus);
double dt_locus_gsum (long which, double x, double *param, nr_fmt * nr,
		      timearchive_fmt ** atl, long locus);


double gamma_loci_like(nr_fmt *nr, double denom, double *oparam, double *olparam)
{
  long locus;
  double summ=0.0;
  for(locus=0;locus<nr->world->loci;locus++)
    {
      if (nr->skiploci[locus])
	continue;
      nr->locilikes[locus] = gamma_locus_like(nr,oparam, olparam, locus, denom);      summ += nr->locilikes[locus];
    }
  return summ;
}

double gamma_locus_like(nr_fmt *nr, double *oparam, double *olparam, 
			long locus, double denom)
{
  long r;
  double tempmax = -DBL_MAX;
  double *temp;
  double summ=0.;
  double alpha = oparam[nr->numpop2];
  double theta1 = oparam[0];
  temp = (double *) calloc(GAMMA_INTERVALS, sizeof(double));
  for (r=0;r<GAMMA_INTERVALS;r++)
    {
      set_gamma_param (nr->param, oparam, 
		       nr->lparam, olparam, nr->rate[r], nr);
      temp[r] = nr->probcat[r] + loggf(nr->rate[r], alpha, theta1) - denom +
	(*calc_like)(nr, nr->param, nr->lparam, locus);
      //      printf("%f ",temp[r]);
      if(temp[r]>tempmax)
	tempmax = temp[r];
    }
  //  printf("\n");
  for (r=0; r<GAMMA_INTERVALS; r++)
    summ += exp(temp[r] - tempmax);
  free(temp);
  return log(summ) + tempmax;
}

double loggf(double r, double alpha, double theta1)
{
  return -r * alpha / theta1 + log(r) * (alpha-1.);
}

void gamma_loci_difference(helper_fmt *helper)
{
  long i;
  nr_fmt *nr = helper->nr;
  long size = nr->partsize;
  double mle, high, low, param1;
  double *lparam;
  double *param;
  double *xv = helper->xv;
  double *expxv = helper->expxv;
  double epsilon=EPSILON;
  //  double logepsilon = log(epsilon);
  memset(nr->d,0, size*sizeof(double));
  setdoublevec1d(&lparam, helper->xv, size);
  setdoublevec1d(&param, helper->expxv, size);
  mle = gamma_loci_like(nr, helper->weight, param, lparam);
  for(i=0;i<size;i++)
    {
      memcpy (param, expxv, sizeof (double) * size);
      memcpy (lparam, xv, sizeof (double) * size);
      param1 = (param[i] += epsilon);
      lparam[i] = log(param[i]);
      high = gamma_loci_like(nr, helper->weight, param, lparam);
      param[i] -= 2. * epsilon;
      if(param[i]<0.0)
	{
	  low = mle;
	  param[i]=expxv[i];
	}
      else
	{
	  lparam[i] = log(param[i]);
	  low = gamma_loci_like(nr, helper->weight, param, lparam);
	}
      nr->d[i] = (high - low)/(param1 - param[i]);
    }
  free(lparam);
  free(param);
}

void gamma_loci_derivative(helper_fmt *helper)
{
  long locus;
  memset(helper->nr->d,0,helper->nr->partsize*sizeof(double));
  for(locus=0; locus<helper->nr->world->loci; locus++)
    {
      if (helper->nr->skiploci[locus])
	continue;
      //adds up in nr->d
      gamma_locus_derivative(helper,locus);
    }
}

void  gamma_locus_derivative(helper_fmt *helper, long locus)
{
  long r, i, ii,z;
  long double **temp, *tempmax;
  long double ll;
  long double sumi;
  int **stemp;
  //double *summll;
  //double summllsum;
  //double summllmax;
  double rate;
  double lograte;
  double tmp;  
  double denom = helper->weight;
  nr_fmt *nr = helper->nr;
  double alpha = helper->expxv[nr->numpop2];
  double theta1 = helper->expxv[0];
  long partsize = nr->partsize;
  //summll = (double *) calloc(GAMMA_INTERVALS, sizeof(double));
  tempmax = (long double *) calloc(partsize, sizeof(long double));
  temp = (long double **) calloc(partsize, sizeof(long double*));
  temp[0] = (long double *) calloc(partsize * GAMMA_INTERVALS, sizeof(long double));
  stemp = (int **) calloc(partsize, sizeof(int*));
  stemp[0] = (int *) calloc(partsize * GAMMA_INTERVALS, sizeof(int));
  tempmax[0]= -DBL_MAX;
  for (r=1;r<partsize;r++)
    {
      temp[r] = temp[0] + r * GAMMA_INTERVALS;  
      stemp[r] = stemp[0] + r * GAMMA_INTERVALS;
      tempmax[r] = -DBL_MAX;
    }
  //summllmax = -DBL_MAX;
  //summllsum=0.0;
  for (r=0;r<GAMMA_INTERVALS;r++)
    {
      rate = nr->rate[r];
      lograte = log(rate);
      set_gamma_param (nr->param, helper->expxv, 
		       nr->lparam, helper->xv, rate, nr);
      ll = nr->probcat[r] + (*calc_like) (nr, nr->param, nr->lparam, locus);
      ll += -rate * alpha / theta1  + lograte * (alpha - 1.) - denom;
      //summll[r] = ll;
      //if(ll>summllmax)
      //summllmax = ll;
      gamma_parts_gradient(rate, helper->expxv, nr, locus);//changes nr->parts
      //first of the derivatives: theta1
      tmp = rate * alpha / (theta1 * theta1) + nr->parts[0];
      stemp[0][r] =  tmp < 0. ? -1 : 1;
      temp[0][r] = ll + log(fabs(tmp));
      if(tempmax[0] < temp[0][r])
      	tempmax[0] = temp[0][r];
      //all others except alpha
      for(i=1;i<nr->numpop2;i++)
      {
	stemp[i][r] = (nr->parts[i] < 0.) ? -1 : 1; 
	temp[i][r] =  ll + log(fabs(nr->parts[i]));
	if(tempmax[i] < temp[i][r])
	  tempmax[i] = temp[i][r];
      }
      // last derivative: alpha
      tmp =  - rate / theta1 + lograte;
      stemp[i][r] =  tmp < 0. ? -1 : 1;
      temp[i][r] = ll + log(fabs(tmp));
      if(tempmax[i] < temp[i][r])
	tempmax[i] = temp[i][r];
    }
  //for (r=0;r<GAMMA_INTERVALS;r++)
  //  {
  //    summllsum += exp(summll[r] - summllmax);
  //  }
  //summllsum = log(summllsum) + summllmax; 
  z=0;
  for(ii=0; ii < nr->partsize - nr->profilenum; ii++)
    {
      i = (nr->profilenum > 0) ? nr->indeks[z++] : ii; 
      sumi = 0;
      for (r=0;r<GAMMA_INTERVALS;r++)
	sumi += ((double) stemp[i][r]) *  exp(temp[i][r] - tempmax[i] );
      if(i==0)
	{
	  //  printf("summll=%f locilikes[%li]=%f\n",summllsum,locus,nr->locilikes[locus]);
	  sumi = sumi * exp(tempmax[i] - nr->locilikes[locus]) - alpha/theta1;
	}
      else 
	{
	  if(i==nr->numpop2)
	    {
	      sumi = sumi * exp(tempmax[i] - nr->locilikes[locus])
		- polygamma(0, alpha) 
		- log(theta1/alpha) + 1.;
	    }
	  else
	    sumi = sumi * exp(tempmax[i] - nr->locilikes[locus]);
	}
      nr->d[ii] += sumi;  
    }
  free(temp[0]);
  free(temp);
  free(stemp[0]);
  free(stemp);
  //free(summll);
}

void gamma_parts_gradient(double x, double *param, nr_fmt *nr, long locus)
{
  long g;

  //  double copysum=0;
  long pop, i;
  double expapg;//, summ;
  //double summm = 0.0;
  long msta, msto;
  double waitmig;
  double pointmig;
  tarchive_fmt *tl;
  double *part;//, df = 0.0;
  //  double value ;
  double theta1 = param[0];
  double *geo = nr->world->data->geo;
  long rep;
  double theta_ratio = theta1 / x;
  part = (double *) malloc(sizeof(double)*nr->partsize);
  memset(nr->parts, 0, sizeof(double)*nr->partsize);
  for (rep = nr->repstart; rep < nr->repstop; rep++)
    {
      for (g = 0; g < nr->atl[rep][locus].T; g++)
	{
	  if (nr->apg[rep][locus][g] > -40.)
	    {
	      tl = nr->atl[rep][locus].tl;
	      //copies = (g > 0) ? tl[g].copies : tl[g].copies - 1;
	      expapg = /*copies  */ exp (nr->apg[rep][locus][g]);
	      if(expapg==0.0)
		continue;
	      //      summm += expapg;
	      // population 1 is treated differently
	      part[0] = 0.0;
	      msta = nr->mstart[0];
	      msto = nr->mend[0];
	      for (i = msta; i < msto; i++)
		{
		  part[0] += -geo[i] * param[i] * tl[g].km[0] / x + 
		    tl[g].mindex[i] / theta1;
		  if (param[i] > 0.)
		    part[i] = ((tl[g].mindex[i] / param[i]) 
			       - geo[i] * tl[g].km[0] 
			       * theta_ratio);
		}
	      // all other populations
	      for (pop = 1; pop < nr->numpop; pop++)
		{
		  part[pop] = -tl[g].p[pop] / param[pop] + 
		    tl[g].kt[pop] * theta1 / (x * param[pop] * param[pop]);
		  
		  msta = nr->mstart[pop];
		  msto = nr->mend[pop];
		  //		  z = 0;
		  waitmig = 0;
		  pointmig = 0;
		  for (i = msta; i < msto; i++)
		    {
		      waitmig += geo[i] * param[i];
		      pointmig += tl[g].mindex[i];
		      if (param[i] > 0.)
			part[i] = ((tl[g].mindex[i] / param[i]) 
				   - geo[i] * tl[g].km[pop] 
				   * theta_ratio);
		    }
		  part[0] += pointmig/theta1 - waitmig * tl[g].km[pop] / x +
		    tl[g].p[pop] / theta1 - tl[g].kt[pop]/(param[pop]*x);
		}
	      for(pop=0;pop<nr->world->numpop2;pop++)
		nr->parts[pop] += expapg * part[pop]; 
	    }
	}
    }
  for(pop=0;pop<nr->world->numpop2;pop++)
    nr->parts[pop] /= nr->PGC[locus]; 
  // printf("nr->PGC[%li]=%f == %f\n",locus,nr->PGC[locus],summm);
  free(part);
}
